#include <windows.h>
#include <stdio.h>
#include "wilrapi.h"
/*
CeTransFiles
BOOL CeTransFiles(LPCTSTR pcfilepath,LPCWSTR newfilepath,BOOL bFailIfExists);
       pcfilepath     source file path
       newfilepath    destination file path
       bFailIfExists  control for exist of destination file
       return success(not FALSE) or failure(FALSE)

example:
result = CeTransFiles(TEXT("C:\\My Documents\\test.arh"),L"\\Windows\\test.arh",FALSE);
*/

#define	CETRANSFILES_COPY_SIZE	(32*1024)

BOOL CeTransFiles(LPCTSTR pcfilepath, LPCWSTR newfilepath, BOOL bFailIfExists)
{
	SECURITY_ATTRIBUTES stSecurity;
	HANDLE	hDesktop;
	HANDLE	hMobile;
	BYTE	byCopyBuffer[CETRANSFILES_COPY_SIZE];
	BOOL	bCopy;
	DWORD	dwReadSize;
	DWORD	dwWriteSize;
	FILETIME	ftCreationTime;
	FILETIME	ftLastAccessTime;
	FILETIME	ftLastWriteTime;
	BOOL	bRet;
	DWORD	dwAttr;

	/* Open Desktop side */
	stSecurity.nLength = sizeof(SECURITY_ATTRIBUTES);
	stSecurity.lpSecurityDescriptor = NULL;
	stSecurity.bInheritHandle = TRUE;
	hDesktop = CreateFile(
		pcfilepath,
		GENERIC_READ,
		FILE_SHARE_READ,
		&stSecurity,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_READONLY,
		NULL
		);
	if (hDesktop == FALSE) {
		return FALSE;
	}

	/* Open Mobile side */
	hMobile = CeCreateFile(
		newfilepath,
		GENERIC_WRITE,
		FILE_SHARE_WRITE,
		&stSecurity,
		(bFailIfExists == FALSE) ? CREATE_ALWAYS : CREATE_NEW,
		FILE_ATTRIBUTE_ARCHIVE,
		NULL
		);
	if (hMobile == FALSE) {
		CloseHandle(hDesktop);
		return FALSE;
	}

	
	for (;;) {

		/* Operation CANCEL check point */
		/* and progress bar control */

		/* Read Desktop side */
		bCopy = ReadFile(hDesktop, byCopyBuffer,
		                 sizeof(byCopyBuffer), &dwReadSize, NULL);
#ifdef _DEBUG
printf("ReadFile() = %d, %d.\n",bCopy,dwReadSize);
#endif
		if (bCopy == FALSE)
			break;
		if (dwReadSize == 0)
			break;
		/* Write Mobile side */
		bCopy = CeWriteFile(hMobile, byCopyBuffer,
		                    dwReadSize, &dwWriteSize, NULL);
#ifdef _DEBUG
printf("WriteFile() = %d, %d.\n",bCopy,dwWriteSize);
#endif
		if (bCopy == FALSE)
			break;
		if (dwReadSize != dwWriteSize)
			break;
	}
	/* Copy error check */
	if ((bCopy == FALSE) || (dwReadSize != 0)) {
		CloseHandle(hDesktop);
		CeCloseHandle(hMobile);
		return FALSE;
	}

	/* Get Desktop side timestamp */
	bRet = GetFileTime(hDesktop,
	                   &ftCreationTime, &ftLastAccessTime, &ftLastWriteTime);
#ifdef _DEBUG
printf("GetFileTime() = %d.\n",bRet);
#endif
	/* Set Mobile side timestamp */
	if (bRet != FALSE)
		bRet = CeSetFileTime(hMobile,
		                     &ftCreationTime, &ftLastAccessTime, &ftLastWriteTime);
#ifdef _DEBUG
printf("CeSetFileTime() = %d.\n",bRet);
#endif

	bRet = CloseHandle(hDesktop);
#ifdef _DEBUG
printf("CloseHandle() = %d.\n",bRet);
#endif
	bRet = CeCloseHandle(hMobile);
#ifdef _DEBUG
printf("CeCloseHandle() = %d.\n",bRet);
#endif

	/* Get Desktop side fileattributes */
	dwAttr = GetFileAttributes(pcfilepath);
	/* Set Mobile side fileattributes */
	if (dwAttr != 0xFFFFFFFF)
		bRet = CeSetFileAttributes(newfilepath, dwAttr);
#ifdef _DEBUG
printf("dwAttr = %X, bRet = %d.\n",dwAttr,bRet);
#endif

	return TRUE;
}
void
main(int argc, char *argv[])
{
	BOOL	result;
	BYTE	Uuid[16];
	PROCESS_INFORMATION	ProcessInformation;
	int	r1;

	CeRapiInit();
	result = CeTransFiles(argv[1],L"\\Windows\\test.arh",FALSE);
	printf("result = %d.\n",result);
	memset(Uuid,0x00,sizeof(Uuid));
	result = WilGetUuid(Uuid);
	printf("GetUuid() = %d.\n",result);
	printf("Uuid =");
	for (r1 = 0; r1 < sizeof(Uuid); r1++)
		printf(" %02X",Uuid[r1]);
	printf(".\n");
	memset(&ProcessInformation,0x00,sizeof(ProcessInformation));
	result = CeCreateProcess(L"calc.exe",NULL,NULL,NULL,FALSE,0,NULL,NULL,NULL,&ProcessInformation);
	printf("CreateProcess() = %d.\n",result);
	printf("Process Information: hProcess = %X, hThread = %X, dwProcessId = %X, dwThreadId = %X.\n",
	       ProcessInformation.hProcess,ProcessInformation.hThread,
	       ProcessInformation.dwProcessId,ProcessInformation.dwThreadId);
	CeRapiUninit();
}
